@@ -22,6 +22,9 @@
 #ifndef _RATPOISON_DATA_H
 #define _RATPOISON_DATA_H
 
+#include "linkedlist.h"
+#include "number.h"
+
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -29,27 +32,43 @@
 #define FONT_HEIGHT(f) ((f)->max_bounds.ascent + (f)->max_bounds.descent)
 
 #define WIN_EVENTS (StructureNotifyMask | PropertyChangeMask | ColormapChangeMask | FocusChangeMask)
+/* EMPTY is used when a frame doesn't contain a window, or a window
+   doesn't have a frame. Any time a field refers to the number of a
+   window/frame/screen/etc, Use EMPTY to denote a lack there of. */
+#define EMPTY -1
+
+/* Possible values for defaults.window_list_style */
+#define STYLE_ROW    0 
+#define STYLE_COLUMN 1
+
+/* Possible values for defaults.win_name */
+#define WIN_NAME_TITLE 		0
+#define WIN_NAME_RES_CLASS 	1
+#define WIN_NAME_RES_NAME 	2
 
 typedef struct rp_window rp_window;
-typedef struct screen_info screen_info;
+typedef struct rp_screen rp_screen;
 typedef struct rp_action rp_action;
-typedef struct rp_window_frame rp_window_frame;
+typedef struct rp_frame rp_frame;
+typedef struct rp_child_info rp_child_info;
 
-struct rp_window_frame
+struct rp_frame
 {
   int number;
   int x, y, width, height;
-  rp_window *win;
+
+  /* The number of the window that is focused in this frame. */
+  int win_number;
 
   /* For determining the last frame. */
   int last_access;
 
-  rp_window_frame *prev, *next;
+  struct list_head node;
 };
 
 struct rp_window
 {
-  screen_info *scr;
+  rp_screen *scr;
   Window w;
   int number;
   int state;
@@ -85,12 +104,12 @@ struct rp_window
   /* A window can be visible inside a frame but not the frame's
      current window. This keeps track of what frame the window was
      mapped into. */
-  rp_window_frame *frame;
+  int frame_number;
 
-  rp_window *next, *prev;  
+  struct list_head node;
 };
 
-struct screen_info
+struct rp_screen
 {
   GC normal_gc;
   XWindowAttributes root_attr;
@@ -105,11 +124,14 @@ struct screen_info
 
   /* A list of frames that may or may not contain windows. There should
      always be one in the list. */
-  rp_window_frame *rp_window_frame_sentinel;
+  struct list_head frames;
 
-  /* Pointer to the currently focused frame. One for each screen so
+  /* Keep track of which numbers have been given to frames. */
+  struct numset *frames_numset;
+
+  /* The number of the currently focused frame. One for each screen so
      when you switch screens the focus doesn't get frobbed. */
-  rp_window_frame *rp_current_frame;
+  int current_frame;
 };
 
 struct rp_action
@@ -144,6 +166,7 @@ struct rp_defaults
   int bar_border_width;
   
   int frame_indicator_timeout;
+  int frame_resize_unit;
 
   int padding_left;
   int padding_right;
@@ -160,33 +183,56 @@ struct rp_defaults
   int win_name;
 
   int startup_message;
+
+  /* Decides whether the window list is displayed in a row or a
+     column. */
+  int window_list_style;
+
+  /* Pointer warping toggle. */
+  int warp;
 };
 
+/* Information about a child process. */
+struct rp_child_info
+{
+  /* The command that was executed. */
+  char *cmd;
+
+  /* PID of the process. */
+  int pid;
+
+  /* Return status when the child process finished. */
+  int status;
+
+  /* When this is != 0 then the process finished. */
+  int terminated;
+
+  /* This structure can exist in a list. */
+  struct list_head node;
+};
+
+/* Each child process is stored in this list. spawn, creates a new
+   entry in this list, the SIGCHLD handler sets child.terminated to be
+   true and handle_signals in events.c processes each terminated
+   process by printing a message saying the process ended and
+   displaying it's exit code. */
+extern struct list_head rp_children;
+
 extern struct rp_defaults defaults;
 
 /* The prefix key also known as the command character under screen. */
 extern struct rp_key prefix_key;
 
-/* These _sentinel pointers point to a special rp_window whose next
-   pointer points to the head of the list and whose prev pointer
-   points to the tail. This is done to simplify removing an element
-   from the list: since there are no head and tail global variable
-   pointers that need to be updated, functions that manipulate
-   elements in a list don't need to know what list they belong to. The
-   last element's next pointer points to the sentinel as does the
-   first element's prev pointer. An empty list is represented by the
-   sentinel's prev and next pointers pointing to itself. */
-
 /* A list of mapped windows. These windows show up in the window
    list and have a number assigned to them. */
-extern rp_window *rp_mapped_window_sentinel;
+extern struct list_head rp_mapped_window;
 
 /* A list of unmapped windows. These windows do not have a number
    assigned to them and are not visible/active. */
-extern rp_window *rp_unmapped_window_sentinel;
+extern struct list_head rp_unmapped_window;
 
 extern int rp_current_screen;
-extern screen_info *screens;
+extern rp_screen *screens;
 extern int num_screens;
 
 extern XEvent rp_current_event;
@@ -196,6 +242,7 @@ extern Atom rp_command;
 extern Atom rp_command_request;
 extern Atom rp_command_result;
 
+extern Atom wm_name;
 extern Atom wm_state;
 extern Atom wm_change_state;
 extern Atom wm_protocols;
@@ -247,6 +294,7 @@ extern struct modifier_info rp_modifier_info;
 extern int alarm_signalled;
 extern int kill_signalled;
 extern int hup_signalled;
+extern int chld_signalled;
 
 /* rudeness levels */
 extern int rp_honour_transient_raise;
@@ -257,4 +305,7 @@ extern int rp_honour_normal_map;
 /* Keep track of X11 error messages. */
 extern char *rp_error_msg;
 
+/* Number sets for windows. */
+extern struct numset *rp_window_numset;
+
 #endif /* _RATPOISON_DATA_H */
