@@ -33,18 +33,27 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <string.h>
+#include <sys/wait.h>
 
 #include "ratpoison.h"
 
-static void init_screen (screen_info *s, int screen_num);
+/* Several systems seem not to have WAIT_ANY defined, so define it if
+   it isn't. */
+#ifndef WAIT_ANY
+# define WAIT_ANY -1
+#endif
+
+static void init_screen (rp_screen *s, int screen_num);
 
 int alarm_signalled = 0;
 int kill_signalled = 0;
 int hup_signalled = 0;
+int chld_signalled = 0;
 int rat_x;
 int rat_y;
 int rat_visible = 1;		/* rat is visible by default */
 
+Atom wm_name;
 Atom wm_state;
 Atom wm_change_state;
 Atom wm_protocols;
@@ -57,10 +66,11 @@ Atom rp_command_request;
 Atom rp_command_result;
 
 int rp_current_screen;
-screen_info *screens;
+rp_screen *screens;
 int num_screens;
 Display *dpy;
 
+LIST_HEAD (rp_children);
 struct rp_defaults defaults;
 
 int ignore_badwindow = 0;
@@ -84,9 +94,11 @@ static struct option ratpoison_longopts[] =
   { {"help", 	no_argument, 		0, 	'h'},
     {"version", no_argument, 		0, 	'v'},
     {"command", required_argument, 	0, 	'c'},
+    {"display", required_argument, 	0, 	'd'},
+    {"screen",  required_argument,      0,      's'},
     {0, 	0, 			0, 	0} };
 
-static char ratpoison_opts[] = "hvc:";
+static char ratpoison_opts[] = "hvc:d:s:";
 
 void
 fatal (const char *msg)
@@ -110,7 +122,7 @@ xrealloc (void *ptr, size_t size)
   register void *value = realloc (ptr, size);
   if (value == 0)
     fatal ("Virtual memory exhausted");
-  PRINT_DEBUG("realloc: %d\n", size);
+  PRINT_DEBUG (("realloc: %d\n", size));
   return value;
 }
 
@@ -201,17 +213,53 @@ alrm_handler (int signum)
   alarm_signalled++;
 }
 
+void
+chld_handler (int signum)
+{
+  int pid, status, serrno;
+  serrno = errno;
+  rp_child_info *cur;
+
+  while (1)
+    {
+      pid = waitpid (WAIT_ANY, &status, WNOHANG);
+      if (pid <= 0)
+	break;
+
+      PRINT_DEBUG(("Child status: %d\n", WEXITSTATUS (status)));
+
+      /* Find the child and update its structure. */
+      list_for_each_entry (cur, &rp_children, node)
+	{
+	  if (cur->pid == pid)
+	    {
+	      cur->terminated = 1;
+	      cur->status = WEXITSTATUS (status);
+	      break;
+	    }
+	}
+
+      chld_signalled = 1;
+    }
+  errno = serrno;  
+}
+
 int
 handler (Display *d, XErrorEvent *e)
 {
   char error_msg[100];
 
-  if (e->request_code == X_ChangeWindowAttributes && e->error_code == BadAccess) {
-    fprintf(stderr, "ratpoison: There can be only ONE.\n");
-    exit(EXIT_FAILURE);
-  }  
+  if (e->request_code == X_ChangeWindowAttributes && e->error_code == BadAccess) 
+    {
+      fprintf(stderr, "ratpoison: There can be only ONE.\n");
+      exit(EXIT_FAILURE);
+    }  
 
+#ifdef IGNORE_BADWINDOW
+  return 0;
+#else
   if (ignore_badwindow && e->error_code == BadWindow) return 0;
+#endif
 
   XGetErrorText (d, e->error_code, error_msg, sizeof (error_msg));
   fprintf (stderr, "ratpoison: ERROR: %s!\n", error_msg);
@@ -236,7 +284,7 @@ set_sig_handler (int sig, void (*action)(int))
   /* check setting for sig */
   if (sigaction (sig, NULL, &act)) 
     {
-      PRINT_ERROR ("Error %d fetching SIGALRM handler\n", errno );
+      PRINT_ERROR (("Error %d fetching SIGALRM handler\n", errno ));
     } 
   else 
     {
@@ -249,7 +297,7 @@ set_sig_handler (int sig, void (*action)(int))
 	  act.sa_flags = 0;
 	  if (sigaction (sig, &act, NULL)) 
 	    {
-	      PRINT_ERROR ("Error %d setting SIGALRM handler\n", errno );
+	      PRINT_ERROR (("Error %d setting SIGALRM handler\n", errno ));
 	    }
 	}
     }
@@ -270,7 +318,9 @@ print_help ()
   printf ("Help for %s %s\n\n", PACKAGE, VERSION);
   printf ("-h, --help            Display this help screen\n");
   printf ("-v, --version         Display the version\n");
-  printf ("-c, --command         Send ratpoison a colon-command\n\n");
+  printf ("-d, --display <dpy>   Set the X display to use\n");
+  printf ("-s, --screen <num>    Only use the specified screen\n");
+  printf ("-c, --command <cmd>   Send ratpoison a colon-command\n\n");
 
   printf ("Report bugs to ratpoison-devel@lists.sourceforge.net\n\n");
 
@@ -310,7 +360,7 @@ read_rc_file (FILE *file)
 	  if (*(line + strlen(line) - 1) == '\n')
 	    *(line + strlen(line) - 1) = '\0';
 
-	  PRINT_DEBUG ("rcfile line: %s\n", line);
+	  PRINT_DEBUG (("rcfile line: %s\n", line));
 
 	  /* do it */
 	  if (*line != '#')
@@ -344,7 +394,7 @@ read_startup_files ()
   homedir = getenv ("HOME");
   if (!homedir)
     {
-      PRINT_ERROR ("ratpoison: $HOME not set!?\n");
+      PRINT_ERROR (("ratpoison: $HOME not set!?\n"));
     }
   else
     {
@@ -354,12 +404,12 @@ read_startup_files ()
       if ((fileptr = fopen (filename, "r")) == NULL)
 	{
 	  /* we probably don't need to report this, its not an error */
-	  PRINT_DEBUG ("ratpoison: could not open %s\n", filename);
+	  PRINT_DEBUG (("ratpoison: could not open %s\n", filename));
 
 	  if ((fileptr = fopen ("/etc/ratpoisonrc", "r")) == NULL)
 	    {
 	      /* neither is this */
-	      PRINT_DEBUG ("ratpoison: could not open /etc/ratpoisonrc\n");
+	      PRINT_DEBUG (("ratpoison: could not open /etc/ratpoisonrc\n"));
 	    }
 	}
 
@@ -426,6 +476,7 @@ init_defaults ()
   defaults.bar_border_width    = 1;
 
   defaults.frame_indicator_timeout = 1;
+  defaults.frame_resize_unit = 10;
 
   defaults.padding_left   = 0;
   defaults.padding_right  = 0;
@@ -443,8 +494,10 @@ init_defaults ()
 
   defaults.window_fmt = xstrdup ("%n%s%t");
 
-  defaults.win_name = 0;
+  defaults.win_name = WIN_NAME_TITLE;
   defaults.startup_message = 1;
+  defaults.warp = 1;
+  defaults.window_list_style = STYLE_ROW;
 }
 
 int
@@ -454,6 +507,9 @@ main (int argc, char *argv[])
   int c;
   char **command = NULL;
   int cmd_count = 0;
+  int screen_arg = 0;
+  int screen_num = 0;
+  char *display = NULL;
 
   myargv = argv;
 
@@ -487,12 +543,20 @@ main (int argc, char *argv[])
 	  command[cmd_count] = xstrdup (optarg);
 	  cmd_count++;
 	  break;
+	case 'd':
+	  display = xstrdup (optarg);
+	  break;
+	case 's':
+	  screen_arg = 1;
+	  screen_num = strtol (optarg, NULL, 10);
+	  break;
+	  
 	default:
 	  exit (EXIT_FAILURE);
 	}
     }
 
-  if (!(dpy = XOpenDisplay (NULL)))
+  if (!(dpy = XOpenDisplay (display)))
     {
       fprintf (stderr, "Can't open display\n");
       return EXIT_FAILURE;
@@ -509,7 +573,11 @@ main (int argc, char *argv[])
 
       for (i=0; i<cmd_count; i++)
 	{
-	  send_command (command[i]);
+	  if (screen_arg)
+	    send_command (command[i], screen_num);
+	  else
+	    send_command (command[i], -1);
+
 	  free (command[i]);
 	}
 
@@ -519,6 +587,7 @@ main (int argc, char *argv[])
     }
 
   /* Set our Atoms */
+  wm_name =  XInternAtom(dpy, "WM_NAME", False);
   wm_state = XInternAtom(dpy, "WM_STATE", False);
   wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
   wm_protocols = XInternAtom(dpy, "WM_PROTOCOLS", False);
@@ -532,31 +601,61 @@ main (int argc, char *argv[])
   set_sig_handler (SIGTERM, sighandler);
   set_sig_handler (SIGINT, sighandler);
   set_sig_handler (SIGHUP, hup_handler);
+  set_sig_handler (SIGCHLD, chld_handler);
 
   /* Setup ratpoison's internal structures */
-  init_defaults();
-  init_numbers ();
+  init_defaults ();
+  init_window_stuff ();
 
-  /* Initialize the screens */
+  /* Get the number of screens */
   num_screens = ScreenCount (dpy);
-  screens = (screen_info *)xmalloc (sizeof (screen_info) * num_screens);
-  PRINT_DEBUG ("%d screens.\n", num_screens);
 
-  for (i=0; i<num_screens; i++)
+  /* make sure the screen specified is valid. */
+  if (screen_arg)
     {
-      init_screen (&screens[i], i);
+      if (screen_num < 0 || screen_num >= num_screens)
+	{
+	  fprintf (stderr, "%d is an invalid screen for the display\n", screen_num);
+	  exit (EXIT_FAILURE);
+	}
+
+      /* we're only going to use one screen. */
+      num_screens = 1;
+    }
+
+  /* Initialize the screens */
+  screens = (rp_screen *)xmalloc (sizeof (rp_screen) * num_screens);
+  PRINT_DEBUG (("%d screens.\n", num_screens));
+
+  if (screen_arg)
+    {
+      init_screen (&screens[0], screen_num);
+    }
+  else
+    {
+      for (i=0; i<num_screens; i++)
+	{
+	  init_screen (&screens[i], i);
+	}
     }
 
-  init_window_list ();
   init_frame_lists ();
   update_modifier_map ();
   initialize_default_keybindings ();
 
   /* Scan for windows */
-  rp_current_screen = 0;
-  for (i=0; i<num_screens; i++)
+  if (screen_arg)
     {
-      scanwins (&screens[i]);
+      rp_current_screen = screen_num;
+      scanwins (&screens[0]);
+    }
+  else
+    {
+      rp_current_screen = 0;
+      for (i=0; i<num_screens; i++)
+	{
+	  scanwins (&screens[i]);
+	}
     }
 
   read_startup_files ();
@@ -576,13 +675,13 @@ main (int argc, char *argv[])
 }
 
 static void
-init_rat_cursor (screen_info *s)
+init_rat_cursor (rp_screen *s)
 {
   s->rat = XCreateFontCursor( dpy, XC_icon );
 }
 
 static void
-init_screen (screen_info *s, int screen_num)
+init_screen (rp_screen *s, int screen_num)
 {
   XGCValues gv;
 
@@ -590,10 +689,13 @@ init_screen (screen_info *s, int screen_num)
      there is already a WM running and the X Error handler will catch
      it, terminating ratpoison. */
   XSelectInput(dpy, RootWindow (dpy, screen_num),
-               PropertyChangeMask | ColormapChangeMask
-               | SubstructureRedirectMask | SubstructureNotifyMask );
+	       PropertyChangeMask | ColormapChangeMask
+	       | SubstructureRedirectMask | SubstructureNotifyMask );
   XSync (dpy, False);
 
+  /* Create the numset for the frames. */
+  s->frames_numset = numset_new ();
+
   /* Build the display string for each screen */
   s->display_string = xmalloc (strlen(DisplayString (dpy)) + 21);
   sprintf (s->display_string, "DISPLAY=%s", DisplayString (dpy));
@@ -606,6 +708,8 @@ init_screen (screen_info *s, int screen_num)
 	sprintf(dot, ".%i", screen_num);
     }
 
+  PRINT_DEBUG (("%s\n", s->display_string));
+
   s->screen_num = screen_num;
   s->root = RootWindow (dpy, screen_num);
   s->def_cmap = DefaultColormap (dpy, screen_num);
@@ -639,14 +743,14 @@ init_screen (screen_info *s, int screen_num)
   s->key_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 0, 
 				       WhitePixel (dpy, s->screen_num), 
 				       BlackPixel (dpy, s->screen_num));
-  XSelectInput (dpy, s->key_window, KeyPressMask );
+  XSelectInput (dpy, s->key_window, KeyPressMask | KeyReleaseMask);
   XMapWindow (dpy, s->key_window);
 
   /* Create the input window. */
   s->input_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 
 					 defaults.bar_border_width, 
 					 s->fg_color, s->bg_color);
-  XSelectInput (dpy, s->input_window, KeyPressMask );
+  XSelectInput (dpy, s->input_window, KeyPressMask | KeyReleaseMask);
 
   /* Create the frame indicator window */
   s->frame_window = XCreateSimpleWindow (dpy, s->root, 1, 1, 1, 1, defaults.bar_border_width, 
@@ -660,32 +764,59 @@ init_screen (screen_info *s, int screen_num)
   XSync (dpy, 0);
 }
 
+static void
+free_screen (rp_screen *s)
+{
+  rp_frame *frame;
+  struct list_head *iter, *tmp;
+
+  list_for_each_safe_entry (frame, iter, tmp, &s->frames, node)
+    {
+      frame_free (s, frame);
+    }
+ 
+  XDestroyWindow (dpy, s->bar_window);
+  XDestroyWindow (dpy, s->key_window);
+  XDestroyWindow (dpy, s->input_window);
+  XDestroyWindow (dpy, s->frame_window);
+  XDestroyWindow (dpy, s->help_window);
+
+  XFreeCursor (dpy, s->rat);
+  XFreeColormap (dpy, s->def_cmap);
+  XFreeGC (dpy, s->normal_gc);
+
+  free (s->display_string);
+
+  numset_free (s->frames_numset);
+}
+
 void
 clean_up ()
 {
   int i;
 
+  free_keybindings ();
+  free_aliases ();
+  free_bar ();
+  free_history ();
+
+  free_window_stuff ();
+  
   for (i=0; i<num_screens; i++)
     {
-      XDestroyWindow (dpy, screens[i].bar_window);
-      XDestroyWindow (dpy, screens[i].key_window);
-      XDestroyWindow (dpy, screens[i].input_window);
-      XDestroyWindow (dpy, screens[i].frame_window);
-      XDestroyWindow (dpy, screens[i].help_window);
-
-      XFreeCursor (dpy, screens[i].rat);
-      XFreeColormap (dpy, screens[i].def_cmap);
-      XFreeGC (dpy, screens[i].normal_gc);
+      free_screen (&screens[i]);
     }
+  free (screens);
 
   XFreeFont (dpy, defaults.font);
+  free (defaults.window_fmt);
 
   XSetInputFocus (dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
   XCloseDisplay (dpy);
 }
 
-/* Given a root window, return the screen_info struct */
-screen_info *
+/* Given a root window, return the rp_screen struct */
+rp_screen *
 find_screen (Window w)
 {
   int i;
