@@ -24,9 +24,12 @@
 
 #include "data.h"
 
+void clear_unmanaged_list ();
+char *list_unmanaged_windows ();
+void add_unmanaged_window (char *name);
 int unmanaged_window (Window w);
-screen_info* current_screen ();
-void scanwins(screen_info *s);
+rp_screen* current_screen ();
+void scanwins(rp_screen *s);
 void unmanage (rp_window *w);
 int update_window_name (rp_window *win);
 void update_normal_hints (rp_window *win);
