@@ -22,8 +22,25 @@
 #ifndef _RATPOISON_NUMBER_H
 #define _RATPOISON_NUMBER_H 1
 
-int get_unique_window_number ();
-void return_window_number (int n);
-void init_numbers ();
+/* Keep track of a set of numbers. For frames and windows. */
+struct numset
+{
+  /* A list of the numbers taken. */
+  int *numbers_taken;
+
+/* the number of numbers currently stored in the numbers_taken
+   array. */
+  int num_taken;
+
+/* the size of the numbers_taken array. */
+  int max_taken;
+};
+
+struct numset *numset_new ();
+void numset_free (struct numset *ns);
+void numset_release (struct numset *ns, int n);
+int numset_request (struct numset *ns);
+int numset_add_num (struct numset *ns, int n);
+void numset_clear (struct numset *ns);
 
 #endif /* ! _RATPOISON_NUMBER_H */
