@@ -31,17 +31,26 @@
 #include <stdarg.h>
 #include <X11/Xlib.h>
 
-/* Some error reporting macros */
-#define PRE_PRINT_LOCATION fprintf (stderr, "%s:%s():%d: ", __FILE__, __FUNCTION__, __LINE__);
-#define PRINT_ERROR(format, args...) \
-        { fprintf (stderr, PACKAGE ":error -- "); PRE_PRINT_LOCATION; fprintf (stderr, format, ## args); }
+/* Helper macro for error and debug reporting. */
+#define PRINT_LINE(type) printf (PACKAGE ":%s:%d: %s: ",__FILE__,  __LINE__, #type)
+
+/* Error and debug reporting macros. */
+#define PRINT_ERROR(fmt)			\
+do {						\
+  PRINT_LINE (error);				\
+  printf fmt;					\
+  fflush (stdout);				\
+} while (0)
 
-/* Some debugging macros */
 #ifdef DEBUG
-#  define PRINT_DEBUG(format, args...) \
-          { fprintf (stderr, PACKAGE ":debug -- "); PRE_PRINT_LOCATION; fprintf (stderr, format, ## args); }
+#define PRINT_DEBUG(fmt)			\
+do {						\
+  PRINT_LINE (debug);				\
+  printf fmt;					\
+  fflush (stdout);                              \
+} while (0)
 #else
-#  define PRINT_DEBUG(format, args...) 
+#define PRINT_DEBUG(fmt)
 #endif /* DEBUG */
 
 extern XGCValues gv;
@@ -50,7 +59,7 @@ extern XGCValues gv;
 
 #include "data.h"
 #include "manage.h"
-#include "list.h"
+#include "window.h"
 #include "bar.h"
 #include "events.h"
 #include "number.h"
@@ -59,9 +68,11 @@ extern XGCValues gv;
 #include "communications.h"
 #include "sbuf.h"
 #include "split.h"
+#include "frame.h"
+#include "screen.h"
 
 void clean_up ();
-screen_info *find_screen (Window w);
+rp_screen *find_screen (Window w);
 
 void read_rc_file (FILE *file);
 
